
import datetime


from cloud_governance.cloud_resource_orchestration.aws.long_run.monitor_long_run import MonitorLongRun
from cloud_governance.cloud_resource_orchestration.aws.long_run.tag_long_run import TagLongRun
from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.main.environment_variables import environment_variables


class EC2LongRun:
    """
    This class tag & monitor the LongRun EC2 instances.
    User Steps:
    1. Create a Jira Issue in Clouds portal, store the JiraId
    2. Create the EC2 instance, tag JiraId
    CI Steps:
    1. CI Look the instances which are tagged with JiraId
    2. Checks the JiraId had manager approval in the data
    3. If manger approval, append LongRun tags ( Project, LongRunDays, ApprovedManager )
    """

    def __init__(self, region_name: str = ''):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__region_name = region_name if region_name else self.__environment_variables_dict.get('AWS_DEFAULT_REGION')
        self.__tag_long_run = TagLongRun(region_name=region_name)
        self.__monitor_long_run = MonitorLongRun(region_name=region_name)
        self.__ec2_operations = EC2Operations()
        self.__es_upload = ElasticUpload()
        self.__es_index = self.__environment_variables_dict.get('es_index')
        self.__account = self.__environment_variables_dict.get('account')
        self.__jira_queue = self.__environment_variables_dict.get('JIRA_QUEUE')

    def update_es_data(self, cost_estimation: float, instances: list, jira_id: str):
        """This method update the es_data"""
        es_data = self.__es_upload.elastic_search_operations.get_es_data_by_id(id=jira_id, index=self.__es_index)
        source = es_data.get('_source')
        instance_ids = source.get('instance_ids', [])
        update_es_data = {'cost_estimation': cost_estimation,
                          'long_run_days': int(instances[0].get('long_run_days')),
                          'total_instances': len(instances)}
        running_days = 0
        total_price = 0
        for instance in instances:
            running_days = max(running_days, instance.get('instance_running_days'))
            total_price += instance.get('total_run_price')
            instance_id = instance.get('instance_id')
            instance_ids.append(instance_id)
            source_instance_id = source.get(instance_id)
            update_es_data[instance_id] = \
                {'total_run_price': round(source_instance_id.get('total_run_price') + instance.get('total_run_price'),
                                         3)}
            update_es_data[instance_id].update({'total_run_hours': round(
                source_instance_id.get('total_run_hours') + instance.get('total_run_hours'), 3)})
            update_es_data[instance_id].update({'last_saved_time': instance.get('last_saved_time')})
            update_es_data[instance_id].update({'instance_state': instance.get('instance_state')})
            update_es_data[instance_id].update({'instance_running_days': instance.get('instance_running_days')})
            update_es_data[instance_id].update(
                {'ebs_cost': round(float(instance.get('ebs_cost') + source_instance_id.get('ebs_cost')), 3)})
        update_es_data['running_days'] = running_days
        update_es_data['total_run_price'] = float(source.get('total_run_price')) + total_price
        update_es_data['remaining_days'] = int(update_es_data.get('long_run_days')) - running_days
        update_es_data['timestamp'] = datetime.datetime.utcnow()
        update_es_data['instance_ids'] = list(set(instance_ids))
        self.__es_upload.elastic_search_operations.update_elasticsearch_index(index=self.__es_index, id=jira_id,
                                                                              metadata=update_es_data)
        logger.info(f'Updated the jira-id: {jira_id} data : {update_es_data}')

    def upload_new_es_data(self, jira_id: str, instances: list, cost_estimation: float):
        """This method upload the new es_data"""
        es_data = {'jira_id': jira_id, 'cloud_name': 'aws', 'account_name': self.__account,
                   'user': instances[0].get('user'), 'long_run_days': instances[0].get('long_run_days'),
                   'owner': instances[0].get('owner'), 'approved_manager': instances[0].get('approved_manager'),
                   'user_manager': instances[0].get('manager'), 'region_name': self.__region_name,
                   'project': instances[0].get('project'), 'cost_estimation': cost_estimation,
                   'jira_id_state': 'in-progress',
                   'total_instances': len(instances)}
        running_days = 0
        total_price = 0
        for instance in instances:
            running_days = max(running_days, instance.get('instance_running_days'))
            total_price += instance.get('total_run_price')
            es_data.setdefault('instance_ids', []).append(instance.get('instance_id'))
            instance.pop('user')
            instance.pop('long_run_days')
            instance.pop('owner')
            instance.pop('approved_manager')
            instance.pop('project')
            instance.pop('manager')
            es_data[instance.get('instance_id')] = instance
        es_data['running_days'] = running_days
        es_data['remaining_days'] = int(es_data.get('long_run_days')) - running_days
        es_data['total_run_price'] = round(total_price, 3)
        es_data['timestamp'] = datetime.datetime.utcnow()
        self.__es_upload.es_upload_data(items=[es_data], es_index=self.__es_index, set_index='jira_id')
        logger.info(f'Uploaded data to the es index {self.__es_index}')

    @logger_time_stamp
    def prepare_to_upload_es(self, upload_data: dict):
        """
        This method beautify and upload data to ES
        """
        for jira_id, instances in upload_data.items():
            issue_description = self.__tag_long_run.jira_operations.get_issue_description(jira_id=jira_id, state='any')
            cost_estimation = float(issue_description.get('CostEstimation', 0))
            cost_estimation += float(
                self.__tag_long_run.jira_operations.get_issue_sub_tasks_cost_estimation(jira_id=jira_id))
            if self.__jira_queue not in jira_id:
                jira_id = f"{self.__jira_queue}-{jira_id}"
            if self.__es_upload.elastic_search_operations.verify_elastic_index_doc_id(index=self.__es_index,
                                                                                      doc_id=jira_id):
                self.update_es_data(cost_estimation=cost_estimation, instances=instances, jira_id=jira_id)
            else:
                self.upload_new_es_data(cost_estimation=cost_estimation, instances=instances, jira_id=jira_id)

    def __long_run(self):
        """
        This method start the long run process
        1. tag the instances which have tag JiraId
        2. Monitor the long_run instances based on LongRunDays
        """
        tag_response = self.__tag_long_run.run()
        if tag_response:
            logger.info(f'Tags are added to the JiraId tag instances: {tag_response}')
        monitor_response = self.__monitor_long_run.run()
        if monitor_response:
            self.prepare_to_upload_es(monitor_response)

    def run(self):
        """
        This method run the long run methods
        """
        self.__long_run()




from cloud_governance.cloud_resource_orchestration.aws.long_run.monitor_in_progress_issues import MonitorInProgressIssues
from cloud_governance.cloud_resource_orchestration.aws.long_run.tag_long_run import TagLongRun
from cloud_governance.cloud_resource_orchestration.common.ec2_monitor_operations import EC2MonitorOperations
from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.common.jira.jira import logger
from cloud_governance.common.jira.jira_operations import JiraOperations
from cloud_governance.common.ldap.ldap_search import LdapSearch
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix
from cloud_governance.main.environment_variables import environment_variables

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations


class MonitorLongRun:
    """This class monitors the long run instances and returns the data"""

    FIRST_ALERT: int = 5
    SECOND_ALERT: int = 3
    DEFAULT_ADMINS = ['athiruma@redhat.com', 'ebattat@redhat.com', 'natashba@redhat.com']
    HOURS_IN_SECONDS = 3600
    JIRA_ID = 'JiraId'

    def __init__(self, region_name: str = ''):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__region_name = region_name if region_name else self.__environment_variables_dict.get('AWS_DEFAULT_REGION')
        self.__ec2_operations = EC2Operations(region=self.__region_name)
        self.__ldap_search = LdapSearch(ldap_host_name=self.__environment_variables_dict.get('LDAP_HOST_NAME'))
        self.__tag_long_run = TagLongRun(region_name=self.__region_name)
        self.jira_operations = JiraOperations()
        self.__es_upload = ElasticUpload()
        self.__es_index = self.__environment_variables_dict.get('es_index')
        self.__mail_message = MailMessage()
        self.__postfix = Postfix()
        self.__ec2_monitor_operations = EC2MonitorOperations(region_name=self.__region_name)
        self.monitor_in_progress = MonitorInProgressIssues(region_name=self.__region_name)
        self.__jira_queue = self.__environment_variables_dict.get('JIRA_QUEUE')

    def __alert_instance_user(self, issues_data: dict):
        """
        This method alert the instance user, if the LongRunDays are running out
        """
        for jira_id, instances in issues_data.items():
            if self.__jira_queue not in jira_id:
                jira_id = f'{self.__jira_queue}-{jira_id}'
            long_run_days = int(instances[0].get('long_run_days'))
            approved_manager = instances[0].get('approved_manager')
            user = instances[0].get('user')
            running_days = 0
            for instance in instances:
                running_days = max(running_days, instance.get('instance_running_days'))
            cc = self.DEFAULT_ADMINS
            if approved_manager:
                cc.append(approved_manager)
            user_details = self.__ldap_search.get_user_details(user_name=user)
            if user_details:
                cc.append(f'{user_details.get("managerId")}@redhat.com')
            if running_days >= long_run_days - self.FIRST_ALERT:
                sub_tasks = self.jira_operations.get_jira_id_sub_tasks(jira_id=jira_id)
                if sub_tasks:
                    self.__tag_long_run.tag_extend_instances(sub_tasks=sub_tasks, jira_id=jira_id)
            subject, body = '', ''
            if running_days == long_run_days - self.FIRST_ALERT:
                subject, body = self.__mail_message.get_long_run_alert(user=user, days=self.FIRST_ALERT, jira_id=jira_id)
            elif running_days == long_run_days - self.SECOND_ALERT:
                subject, body = self.__mail_message.get_long_run_alert(user=user, days=self.FIRST_ALERT, jira_id=jira_id)
            else:
                if running_days >= long_run_days:
                    subject, body = self.__mail_message.get_long_run_expire_alert(user=user, jira_id=jira_id)
            if subject and body:
                self.__postfix.send_email_postfix(subject=subject, to=user, cc=cc, content=body, mime_type='html')

    def monitor_instances(self):
        """
        This method monitoring the LongRun instances which have tag LongRunDays
        """
        jira_id_alerts = {}
        long_run_instances = self.__ec2_monitor_operations.get_instances_by_filtering(tag_key_name='LongRunDays')
        for instance in long_run_instances:
            for resource in instance['Instances']:
                instance_id, instance_type, tags, launch_datetime, instance_state = resource.get('InstanceId'), resource.get('InstanceType'), resource.get('Tags'), resource.get('LaunchTime'), resource.get('State')['Name']
                jira_id = self.__ec2_operations.get_tag_value_from_tags(tag_name=self.JIRA_ID, tags=tags)
                run_hours, last_saved_time = self.__ec2_monitor_operations.get_instance_run_hours(instance=resource, jira_id=jira_id)
                price = self.__ec2_monitor_operations.get_instance_hours_price(instance_type=instance_type, run_hours=run_hours)
                create_time = self.__ec2_monitor_operations.get_attached_time(volume_list=resource.get('BlockDeviceMappings'))
                ebs_cost = self.__ec2_monitor_operations.get_volumes_cost(resource.get('BlockDeviceMappings'))
                running_days = self.__ec2_monitor_operations.calculate_days(launch_date=launch_datetime)
                jira_id_alerts.setdefault(jira_id, []).append({
                    'instance_id': instance_id,
                    'total_run_hours': run_hours,
                    'total_run_price': price,
                    'instance_create_time': create_time,
                    'instance_state': instance_state,
                    'instance_type': instance_type,
                    'last_saved_time': last_saved_time,
                    'jira_id': jira_id,
                    'user': self.__ec2_operations.get_tag_value_from_tags(tag_name='User', tags=tags),
                    'manager': self.__ec2_operations.get_tag_value_from_tags(tag_name='Manager', tags=tags),
                    'approved_manager': self.__ec2_operations.get_tag_value_from_tags(tag_name='ApprovedManager', tags=tags),
                    'long_run_days': self.__ec2_operations.get_tag_value_from_tags(tag_name='LongRunDays', tags=tags),
                    'instance_running_days': running_days,
                    'owner': self.__ec2_operations.get_tag_value_from_tags(tag_name='Owner', tags=tags),
                    'project': self.__ec2_operations.get_tag_value_from_tags(tag_name='Project', tags=tags),
                    'instance_name': self.__ec2_operations.get_tag_value_from_tags(tag_name='Name', tags=tags),
                    'ebs_cost': ebs_cost
                })
        self.__alert_instance_user(issues_data=jira_id_alerts)
        return jira_id_alerts

    def run(self):
        """
        This method run the long run monitoring methods
        """
        response = self.monitor_in_progress.monitor_progress_issues()
        logger.info(f"Closed JiraId's: {response}")
        return self.monitor_instances()



from datetime import datetime, timedelta

import boto3


class CostExplorerOperations:
    """
    This class extracts the price data from AWS Cost Explorer
    """

    START_DAY = 1
    END_DAY = 31
    PURCHASE_OPTIONS = ['On Demand Instances', 'Savings Plans', 'Spot Instances', 'Standard Reserved Instances']

    def __init__(self, ce_client=''):
        self.cost_explorer_client = boto3.client('ce') if not ce_client else ce_client

    def get_cost_by_tags(self, tag: str, granularity: str = 'DAILY', cost_metric: str = 'UnblendedCost',
                         start_date: str = '', end_date: str = ''):
        """
        This method extracts the price by Tag provided
        @return:
        """
        if not start_date and not end_date:
            end_date = datetime.now() + timedelta(self.START_DAY)
            start_date = end_date - timedelta(self.END_DAY)
            start_date = str(start_date.strftime('%Y-%m-%d'))
            end_date = str(end_date.strftime('%Y-%m-%d'))
        if tag.upper() == 'ChargeType'.upper():
            return self.get_cost_and_usage_from_aws(start_date=start_date, end_date=end_date, granularity=granularity,
                                                    GroupBy=[{'Type': 'DIMENSION', 'Key': 'RECORD_TYPE'}])
        elif tag.upper() == 'PURCHASETYPE':
            return self.get_cost_and_usage_from_aws(start_date=start_date, end_date=end_date, granularity=granularity,
                                                    GroupBy=[{'Type': 'DIMENSION', 'Key': 'PURCHASE_TYPE'}])
        else:
            return self.get_cost_and_usage_from_aws(start_date=start_date, end_date=end_date, granularity=granularity,
                                                    cost_metric=cost_metric, GroupBy=[{'Type': 'TAG', 'Key': tag}])

    def get_cost_and_usage_from_aws(self, start_date: str, end_date: str, granularity: str = 'DAILY',
                                    cost_metric: str = 'UnblendedCost', **kwargs):
        """
        This method returns the cost and usage reports
        @param start_date:
        @param end_date:
        @param granularity:
        @param cost_metric:
        @param kwargs:
        @return:
        """
        usage_cost = {}
        response = self.cost_explorer_client.get_cost_and_usage(TimePeriod={
            'Start': start_date,
            'End': end_date
        }, Granularity=granularity, Metrics=[cost_metric], **kwargs)
        usage_cost['GroupDefinitions'] = response.get('GroupDefinitions')
        usage_cost['ResultsByTime'] = response.get('ResultsByTime')
        usage_cost['DimensionValueAttributes'] = response.get('DimensionValueAttributes')
        while response.get('NextPageToken'):
            response = self.cost_explorer_client.get_cost_and_usage(TimePeriod={
                'Start': start_date,
                'End': end_date
            }, Granularity=granularity, Metrics=[cost_metric], NextPageToken=response.get('NextPageToken'), **kwargs)
            usage_cost['ResultsByTime'].extend(response.get('ResultsByTime'))
            usage_cost['DimensionValueAttributes'].extend(response.get('DimensionValueAttributes'))
        return usage_cost

    def get_cost_forecast(self, start_date: str, end_date: str, granularity: str, cost_metric: str, **kwargs):
        """
        This method return the cost forecasting
        @param start_date:
        @param end_date:
        @param granularity:
        @param cost_metric:
        @return:
        """
        return self.cost_explorer_client.get_cost_forecast(
            TimePeriod={
                'Start': start_date,
                'End': end_date
            },
            Granularity=granularity,
            Metric=cost_metric, **kwargs
        )



import calendar
from datetime import datetime, date
import json
import os.path

import pytz

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.clouds.aws.price.price import AWSPrice
from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.main.environment_variables import environment_variables


class EC2MonitorOperations:
    """This class contains the ec2 monitor operations"""

    CURRENT_DAY = datetime.now()
    HOURS_IN_SECONDS = 3600
    DEFAULT_ROUND_VALUE = 3
    HOURS_IN_DAY = 24
    DEFAULT_OS = 'Linux'

    def __init__(self, region_name: str):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__region_name = region_name
        self.__trails_snapshot_time = self.__environment_variables_dict.get('TRAILS_SNAPSHOT_TIME')
        self.__trail_logs = self.get_trail_logs()
        self.__es_upload = ElasticUpload()
        self.__es_instance_data = {}
        self.__aws_price = AWSPrice(region_name=self.__region_name)
        self.__ec2_operations = EC2Operations(region=self.__region_name)

    def get_trail_logs(self):
        """This method get trail logs from the file"""
        if self.__trails_snapshot_time:
            file_path = f'/tmp/{self.__trails_snapshot_time.date()}.json'
            if os.path.exists(file_path):
                with open(file_path) as file:
                    trails = json.load(file)
                    return trails if trails else []
            else:
                raise FileNotFoundError
        return []

    def get_instance_data_in_es(self, jira_id: str, instance_id: str):
        """This method get instance data in es"""
        try:
            es_index = self.__es_upload.es_index
            if self.__es_upload.elastic_search_operations.verify_elastic_index_doc_id(index=es_index, doc_id=jira_id):
                es_data = self.__es_upload.elastic_search_operations.get_es_data_by_id(id=jira_id, index=es_index)
                self.__es_instance_data = es_data.get('_source').get(instance_id)
        except:
            pass

    def get_value_from_es_data(self, name: str):
        """
        This method get the data from the
        """
        if self.__es_instance_data:
            source = self.__es_instance_data
            return source.get(name)
        return ''

    def get_instance_logs(self, instance_id: str, last_saved_time: datetime):
        """This method returns the instance logs"""
        instance_logs = []
        for trail_log in self.__trail_logs:
            for resource in trail_log.get('Resources'):
                if resource.get('ResourceName') == instance_id:
                    event_time = datetime.strptime(trail_log.get('EventTime'), "%Y-%m-%dT%H:%M:%S%z")
                    event_name = trail_log.get('EventName')
                    if event_name == 'TerminateInstances':
                        event_name = 'StopInstances'
                    if last_saved_time:
                        if last_saved_time < event_time:
                            instance_logs.append({'Key': event_name, 'Value': event_time})
                    else:
                        instance_logs.append({'Key': event_name, 'Value': event_time})
        return sorted(instance_logs, key=lambda a: a['Value']) if instance_logs else []

    def get_hours_in_two_date_times(self, time1: datetime, time2: datetime):
        """This method get the difference between two time hours"""
        diff_time = time2 - time1
        hours = diff_time.total_seconds()/self.HOURS_IN_SECONDS
        return round(hours, self.DEFAULT_ROUND_VALUE)

    def process_tails(self, trails: list):
        """
        This method processes the instance start/stop logs and removes consecutive start/stop logs
        """
        if len(trails) == 0 or len(trails) == 1:
            return trails
        processed_trails = [trails[0]]
        prev_instance_state = trails[0]["Key"]
        for idx in range(1, len(trails)):
            curr_instance_state = trails[idx]["Key"]
            if prev_instance_state != curr_instance_state:
                processed_trails.append(trails[idx])
                prev_instance_state = curr_instance_state
        return processed_trails

    def get_run_hours_from_trails(self, trails: list, last_instance_state: str, create_datetime: datetime, launch_time: datetime, trails_snapshot_time: datetime = environment_variables.environment_variables_dict.get('TRAILS_SNAPSHOT_TIME'), last_saved_time: datetime = None, present_state: str = ''):
        """
        This method returns the trail hours
        """
        trails = self.process_tails(trails)
        tzinfo = launch_time.tzinfo
        run_hours = 0
        if not trails:
            if last_instance_state not in ('stopped', 'terminated'):
                if last_saved_time:
                    run_hours += self.get_hours_in_two_date_times(time1=last_saved_time, time2=trails_snapshot_time)
                else:
                    run_hours += self.get_hours_in_two_date_times(time1=launch_time, time2=trails_snapshot_time)
            return run_hours
        start = 0
        end = len(trails) - 1
        if present_state == 'terminated' and len(trails) == 1:
            return 0
        if trails[0].get('Key') == 'StopInstances' and trails[len(trails) - 1].get('Key') == 'StopInstances':
            start += 1
            stop_event_time = trails[0].get('Value').astimezone(tzinfo)
            if last_saved_time:
                run_hours += self.get_hours_in_two_date_times(time1=last_saved_time, time2=stop_event_time)
            else:
                if create_datetime < launch_time < stop_event_time:
                    run_hours += self.get_hours_in_two_date_times(time1=launch_time, time2=stop_event_time)
                else:
                    run_hours += self.get_hours_in_two_date_times(time1=create_datetime, time2=stop_event_time)
        elif trails[0].get('Key') == 'StartInstances' and trails[len(trails) - 1].get('Key') == 'StartInstances':
            end -= 1
            start_event_time = trails[0].get('Value').astimezone(tzinfo)
            run_hours += self.get_hours_in_two_date_times(time1=start_event_time, time2=trails_snapshot_time)
        elif trails[0].get('Key') == 'StopInstances' and trails[len(trails) - 1].get('Key') == 'StartInstances':
            start += 1
            stop_event_time = trails[0].get('Value').astimezone(tzinfo)
            if last_saved_time:
                run_hours += self.get_hours_in_two_date_times(time1=last_saved_time, time2=stop_event_time)
            else:
                if create_datetime < launch_time < stop_event_time:
                    run_hours += self.get_hours_in_two_date_times(time1=launch_time, time2=stop_event_time)
                else:
                    run_hours += self.get_hours_in_two_date_times(time1=create_datetime, time2=stop_event_time)
            end -= 1
            start_event_time = trails[0].get('Value').astimezone(tzinfo)
            run_hours += self.get_hours_in_two_date_times(time1=start_event_time, time2=trails_snapshot_time)
        while start < end and (end - start + 1) > 0:
            start_event_time = trails[start].get('Value').astimezone(tzinfo)
            stop_event_time = trails[start + 1].get('Value').astimezone(tzinfo)
            run_hours += self.get_hours_in_two_date_times(time1=start_event_time, time2=stop_event_time)
            start += 2
        return round(run_hours, self.DEFAULT_ROUND_VALUE)

    def get_last_saved_time(self, tzinfo):
        """This method return the last saved time from the es_data"""
        if self.__es_instance_data:
            saved_time = self.get_value_from_es_data(name='last_saved_time')
            if saved_time:
                return datetime.strptime(saved_time, "%Y-%m-%dT%H:%M:%S%z")
        return None

    def get_attached_time(self, volume_list: list):
        """
        This method return the root volume attached time
        """
        for mapping in volume_list:
            if mapping.get('Ebs').get('DeleteOnTermination'):
                return mapping.get('Ebs').get('AttachTime')
        return ''

    def get_instance_run_hours(self, instance: dict, jira_id: str):
        """This method get the instance run hours"""
        instance_id, instance_state, launch_time = instance.get('InstanceId'), instance.get('State')['Name'], instance.get('LaunchTime')
        launch_time = launch_time.astimezone(pytz.utc)
        tzinfo = launch_time.tzinfo
        self.get_instance_data_in_es(jira_id, instance_id=instance_id)
        last_saved_time = self.get_last_saved_time(tzinfo)
        create_datetime = self.get_attached_time(instance.get('BlockDeviceMappings'))
        last_instance_state = self.get_value_from_es_data(name='instance_state') if self.get_value_from_es_data(name='instance_state') else instance_state
        trails_snapshot_time = self.__trails_snapshot_time if self.__trails_snapshot_time else datetime.now().replace(microsecond=0).astimezone(tzinfo)
        instance_trails = self.get_instance_logs(instance_id, last_saved_time=last_saved_time)
        run_hours = self.get_run_hours_from_trails(trails=instance_trails,
                                                   launch_time=launch_time,
                                                   trails_snapshot_time=trails_snapshot_time,
                                                   last_saved_time=last_saved_time, last_instance_state=last_instance_state,
                                                   create_datetime=create_datetime)
        return run_hours, trails_snapshot_time

    def get_instance_hours_price(self, instance_type: str, run_hours: float):
        """
        This method returns the instance pricing
        """
        region_code = self.__aws_price.get_region_name(self.__region_name)
        price = float(self.__aws_price.get_price(instance=instance_type, region=region_code, os=self.DEFAULT_OS))
        return round(price * run_hours, self.DEFAULT_ROUND_VALUE)

    def calculate_days(self, launch_date: datetime):
        """This method return the no. of days"""
        today = date.today()
        diff_date = today - launch_date.date()
        return diff_date.days

    def get_volumes_cost(self, block_device_mappings: list):
        """This method return the volumes cost from instance attached volumes"""
        volumes_list = []
        for mapping in block_device_mappings:
            if mapping.get('Ebs').get('VolumeId'):
                volumes_list.append(mapping.get('Ebs').get('VolumeId'))
        volumes = self.__ec2_operations.get_volumes(VolumeIds=volumes_list)
        ebs_price = 0
        for volume in volumes:
            create_time = volume.get('CreateTime')
            if self.__trails_snapshot_time:
                create_time = self.__trails_snapshot_time.astimezone(create_time.tzinfo)
            current_datetime = datetime.utcnow().replace(microsecond=0, tzinfo=create_time.tzinfo)
            hours = self.get_hours_in_two_date_times(time1=create_time, time2=current_datetime)
            months = round(hours / calendar.monthrange(current_datetime.year, current_datetime.month)[1], self.DEFAULT_ROUND_VALUE)
            ebs_price += round(self.__aws_price.get_ec2_price(resource='ebs', item_data=volume) * months, self.DEFAULT_ROUND_VALUE)
        return ebs_price

    def get_instances_by_filtering(self, tag_key_name: str):
        """This method get the instances with the tag-key filter"""
        filters = {
            'Filters': [
                {
                    'Name': 'tag-key',
                    'Values': [tag_key_name]
                }
            ]
        }
        return self.__ec2_operations.get_instances(**filters)



import datetime
import json
from json import JSONEncoder
from datetime import datetime, timedelta
import os.path

import pytz

from cloud_governance.cloud_resource_orchestration.aws.long_run.ec2_long_run import EC2LongRun
from cloud_governance.common.clouds.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.jira.jira import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class DateTimeEncoder(JSONEncoder):
    def default(self, o):
        if isinstance(o, datetime):
            return o.isoformat()
        return super(DateTimeEncoder, self).default(o)


class CloudMonitor:
    """
    This class run the short run & long run monitoring
    """

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__region = self.__environment_variables_dict.get('AWS_DEFAULT_REGION', '')
        self.__cloud_name = self.__environment_variables_dict.get('CLOUD_NAME')
        self.__monitor = self.__environment_variables_dict.get('MONITOR')
        self._cloud_trail = CloudTrailOperations(self.__region)
        self.__current_date_time = datetime.utcnow().replace(microsecond=0, tzinfo=pytz.utc)
        environment_variables._environment_variables_dict['TRAILS_SNAPSHOT_TIME'] = self.__current_date_time

    def __write_cloudtrail_logs(self):
        """
        This method fetched the cloudtrail logs per day
        """
        end_time = self.__current_date_time
        start_time = end_time - timedelta(days=1)
        cloud_trail_logs = []
        cloud_trail_logs.extend(self._cloud_trail.get_full_responses(StartTime=start_time, EndTime=end_time, LookupAttributes=[{'AttributeKey': 'EventName', 'AttributeValue': 'StopInstances'}], MaxResults=123))
        cloud_trail_logs.extend(self._cloud_trail.get_full_responses(StartTime=start_time, EndTime=end_time, LookupAttributes=[{'AttributeKey': 'EventName', 'AttributeValue': 'StartInstances'}], MaxResults=123))
        cloud_trail_logs.extend(self._cloud_trail.get_full_responses(StartTime=start_time, EndTime=end_time, LookupAttributes=[{'AttributeKey': 'EventName', 'AttributeValue': 'TerminateInstances'}], MaxResults=123))
        json_data = json.dumps(cloud_trail_logs, cls=DateTimeEncoder)
        path = f'/tmp/{end_time.date()}.json'
        if os.path.exists(path):
            os.remove(path)
            logger.info(f"Deleted the file path:{path}")
        with open(path, 'w') as file:
            file.write(json_data)
            logger.info(path)

    def __delete_cloudtrail_logs(self):
        """
        This method delete the file of cloudtrail logs
        """
        file_path = f'/tmp/{self.__current_date_time}.json'
        if os.path.exists(file_path):
            os.remove(file_path)
            logger.info(f"Deleted the file path:{file_path}")

    @logger_time_stamp
    def aws_cloud_monitor(self):
        """
        This method ture if the cloud name is
        """
        self.__write_cloudtrail_logs()
        if self.__monitor == 'long_run':
            ec2_long_run = EC2LongRun()
            ec2_long_run.run()
        self.__delete_cloudtrail_logs()

    @logger_time_stamp
    def run_cloud_monitor(self):
        """
        This verify the cloud and run the monitor
        """
        if self.__cloud_name.upper() == "AWS".upper():
            logger.info(f'Account = {self.__environment_variables_dict.get("account")}, Region = {self.__region}, Monitoring = {self.__monitor}')
            self.aws_cloud_monitor()

    def run(self):
        """
        This method monitoring the cloud resources
        """
        self.run_cloud_monitor()
